#!/bin/bash
# script de mise en place/mise à jour des installateurs debian/ubuntu
# à partir des netboot.tar.gz
# incorporation des firmwares non libres pour debian
#
# version du 20190806
#
# 2 modes de fonctionnement : verbeux ou silencieux
#
# [TODO] : ubuntu ? bionic ?
# [TODO] : i386 ?
# [TODO] : mettre en paramètres debian/ubuntu et version ?
#
#####

#####
# les variables

mode=""
ladate=$(date +%Y%m%d%H%M%S)
compte_rendu=/root/compte_rendu_install_netboot_${ladate}.txt
#
url_debian="ftp.fr.debian.org/debian"
version_debian="stretch"
depot_firmware_debian="cdimage.debian.org/cdimage/unofficial/non-free/firmware"
#
url_ubuntu="fr.archive.ubuntu.com/ubuntu"
version_ubuntu="bionic"
#
rep_install="/var/sambaedu/unattended/install"

#####
# les fonctions

mode_script()
{
    # gestion du paramètre éventuel $1
    if [ ! "$#" = "0" ]
    then
        case $1 in
            -h)
                # on affiche l'aide
                mode_texte
                exit 0
            ;;
            -v)
                # mode verbeux pour une utilisation manuelle
                mode="verbeux"
            ;;
            -s)
                # mode silencieux pour une utilisation via la crontab
                mode="silence"
            ;;
            *)
                echo "paramètre $1 incorrect"
                # on affiche l'aide
                mode_texte
                exit 2
            ;;
        esac
    else
        # pas de paramètre
        # mode verbeux par défaut
        mode="verbeux"
    fi
}

mode_texte()
{
    echo "Utilisation : $0 [-paramétre]"
    echo "paramètres possibles :"
    echo "   -h : afficher cet aide-mémoire"
    echo "   -s : script silencieux, utilisation avec crontab"
    echo "   -v : script verbeux, utilisation en manuel"
}

tester_repertoires()
{
    # existence de l'arborescence /var/sambaedu/unattended/install/os/iso/
    if [ -e "${rep_install}/os/iso/" ]
    then
        return 0
    else
        # doit-on le créer ?
        return 1
    fi
}

tester_md5sums_firmware()
{
    # téléchargement md5sums dépôt
    wget -q http://$depot_firmware_debian/$version_debian/current/MD5SUMS
    eval md5sums_firmware_depot_${version_debian}=$(cat MD5SUMS | grep "firmware.cpio.gz" | cut -f1 -d" ")
    rm -f MD5SUMS
    # calculer md5sums local
    if [ -e "${rep_install}/os/iso/firmware_${version_debian}.cpio.gz" ]
    then
        eval md5sums_firmware_local_${version_debian}=$(md5sum ${rep_install}/os/iso/firmware_${version_debian}.cpio.gz | cut -f1 -d" ")
    else
        eval md5sums_firmware_local_${version_debian}=""
    fi
    # comparer les md5sums
    eval a='$'md5sums_firmware_local_${version_debian}
    eval b='$'md5sums_firmware_depot_${version_debian}
    if [ "$a" = "$b" ]
    then
        # inutile de télécharger firmware.cpio.gz
        echo "test négatif firmware_${version_debian}.cpio.gz" >> ${compte_rendu}
        return 0
    else
        # il faut télécharger firmware.cpio.gz
        echo "test positif firmware_${version_debian}.cpio.gz" >> ${compte_rendu}
        [ -e "${rep_install}/os/iso/firmware_${version_debian}.cpio.gz" ] && find ${rep_install}/os/iso/firmware_${version_debian}.cpio.gz -delete
        return 1
    fi
}

telecharger_firmware()
{
    # télécharger les firmwares non libres
    if [ "${mode}" = "silence" ]
    then
        wget -P ${rep_install}/os/iso/ -O ${rep_install}/os/iso/firmware_${version_debian}.cpio.gz -q http://${depot_firmware_debian}/${version_debian}/current/firmware.cpio.gz
    else
        wget -P ${rep_install}/os/iso/ -O ${rep_install}/os/iso/firmware_${version_debian}.cpio.gz -q --show-progress http://${depot_firmware_debian}/${version_debian}/current/firmware.cpio.gz
    fi
    echo "firmware_${version_debian}.cpio.gz téléchargé" >> ${compte_rendu}
}

tester_md5sums_netboot()
{
    # 2 arguments :
    # $1 → debian ou ubuntu
    # $2 → i386 ou amd64
    #
    eval url_dists='$'url_${1}
    eval version='$'version_${1}
    # téléchargement md5sums dépôt
    wget -q http://$url_dists/dists/$version/main/installer-${2}/current/images/MD5SUMS
    eval md5sums_netboot_depot_${version}_${2}=$(cat MD5SUMS | grep "./netboot/netboot.tar.gz" | cut -f1 -d" ")
    rm -f MD5SUMS
    # calculer md5sums local
    if [ -e "${rep_install}/os/iso/netboot_${version}_${2}.tar.gz" ]
    then
        eval md5sums_netboot_local_${version}_${2}=$(md5sum ${rep_install}/os/iso/netboot_${version}_${2}.tar.gz | cut -f1 -d" ")
    else
        eval md5sums_netboot_local_${version}_${2}=""
    fi
    # comparer les md5sums
    eval a='$'md5sums_netboot_local_${version}_${2}
    eval b='$'md5sums_netboot_depot_${version}_${2}
    if [ "$a" = "$b" ]
    then
        # inutile de télécharger netboot/netboot.tar.gz
        echo "test négatif netboot_${version}_${2}.tar.gz" >> ${compte_rendu}
        return 0
    else
        # il faut télécharger netboot/netboot.tar.gz
        [ -e "${rep_install}/os/iso/netboot_${version}_${2}.tar.gz" ] && find ${rep_install}/os/iso/netboot_${version}_${2}.tar.gz -delete
        echo "test positif netboot_${version}_${2}.tar.gz" >> ${compte_rendu}
        return 1
    fi
}

telecharger_netboot()
{
    # 2 arguments :
    # $1 → debian ou ubuntu
    # $2 → i386 ou amd64
    #
    # téléchargement des netboot debian/ubuntu 32 bits/64 bits
    eval url_dists='$'url_${1}
    eval version='$'version_${1}
    if [ "${mode}" = "silence" ]
    then
        wget -P ${rep_install}/os/iso/ -O ${rep_install}/os/iso/netboot_${version}_${2}.tar.gz -q http://${url_dists}/dists/${version}/main/installer-${2}/current/images/netboot/netboot.tar.gz
    else
        wget -P ${rep_install}/os/iso/ -O ${rep_install}/os/iso/netboot_${version}_${2}.tar.gz -q --show-progress http://${url_dists}/dists/${version}/main/installer-${2}/current/images/netboot/netboot.tar.gz
    fi
    echo "netboot_${version}_${2}.tar.gz téléchargé" >> ${compte_rendu}
}

extraire_netboot()
{
    # 2 arguments :
    # $1 → debian ou ubuntu
    # $2 → i386 ou amd64
    #
    # extraction des netboot
    eval version='$'version_${1}
    tar xfz ${rep_install}/os/iso/netboot_${version}_${2}.tar.gz -C ${rep_install}/os/ ./${1}-installer/${2}/
    chown -R  www-admin ${rep_install}/os/debian-installer/
    echo "répertoire ${rep_install}/os/${1}-installer/${2}/ en place" >> ${compte_rendu}
}

incorporer_firmware()
{
    # 1 argument :
    # $1 → i386 ou amd64
    cp -p ${rep_install}/os/debian-installer/${1}/initrd.gz ${rep_install}/os/debian-installer/${1}/initrd.gz.orig
    cat ${rep_install}/os/debian-installer/${1}/initrd.gz.orig ${rep_install}/os/iso/firmware_${version_debian}.cpio.gz > ${rep_install}/os/debian-installer/${1}/initrd.gz
    rm -f ${rep_install}/os/debian-installer/${1}/initrd.gz.orig
    echo "firmware debian incorporé pour ${1}" >> ${compte_rendu}
}

#####
# le programme

mode_script $1
echo -e "Compte-rendu de la mise en place ou de la mise à jour de l'installateur debian : ${ladate}\n" > ${compte_rendu}
tester_repertoires
if [ "$?" = "1" ]
then
    echo "il n'y a pas l'arborescence ${rep_install}/os/iso/" | tee -a ${compte_rendu}
    exit 1
fi
tester_md5sums_firmware
if [ "$?" = "1" ]
then
    telecharger_firmware
fi
tester_md5sums_netboot debian amd64
if [ "$?" = "1" ]
then
    telecharger_netboot debian amd64
fi
[ -e "${rep_install}/os/debian-installer/amd64/" ] && find ${rep_install}/os/debian-installer/amd64/ -delete
extraire_netboot debian amd64
incorporer_firmware amd64
exit 0
